# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    VirtualCrossConnectListResponse,
    VirtualCrossConnectCreateResponse,
    VirtualCrossConnectDeleteResponse,
    VirtualCrossConnectUpdateResponse,
    VirtualCrossConnectRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestVirtualCrossConnects:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        virtual_cross_connect = client.virtual_cross_connects.create(
            bgp_asn=1234,
            cloud_provider="aws",
            cloud_provider_region="us-east-1",
            network_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            primary_cloud_account_id="123456789012",
            region_code="ashburn-va",
        )
        assert_matches_type(VirtualCrossConnectCreateResponse, virtual_cross_connect, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        virtual_cross_connect = client.virtual_cross_connects.create(
            bgp_asn=1234,
            cloud_provider="aws",
            cloud_provider_region="us-east-1",
            network_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            primary_cloud_account_id="123456789012",
            region_code="ashburn-va",
            bandwidth_mbps=50,
            name="test interface",
            primary_bgp_key="yFV4wEPtPVPfDUGLWiyQzwga",
            primary_cloud_ip="169.254.0.2",
            primary_telnyx_ip="169.254.0.1",
            secondary_bgp_key="ge1lONeK9RcA83uuWaw9DvZy",
            secondary_cloud_account_id="",
            secondary_cloud_ip="169.254.0.4",
            secondary_telnyx_ip="169.254.0.3",
        )
        assert_matches_type(VirtualCrossConnectCreateResponse, virtual_cross_connect, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.virtual_cross_connects.with_raw_response.create(
            bgp_asn=1234,
            cloud_provider="aws",
            cloud_provider_region="us-east-1",
            network_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            primary_cloud_account_id="123456789012",
            region_code="ashburn-va",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_cross_connect = response.parse()
        assert_matches_type(VirtualCrossConnectCreateResponse, virtual_cross_connect, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.virtual_cross_connects.with_streaming_response.create(
            bgp_asn=1234,
            cloud_provider="aws",
            cloud_provider_region="us-east-1",
            network_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            primary_cloud_account_id="123456789012",
            region_code="ashburn-va",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_cross_connect = response.parse()
            assert_matches_type(VirtualCrossConnectCreateResponse, virtual_cross_connect, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        virtual_cross_connect = client.virtual_cross_connects.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(VirtualCrossConnectRetrieveResponse, virtual_cross_connect, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.virtual_cross_connects.with_raw_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_cross_connect = response.parse()
        assert_matches_type(VirtualCrossConnectRetrieveResponse, virtual_cross_connect, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.virtual_cross_connects.with_streaming_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_cross_connect = response.parse()
            assert_matches_type(VirtualCrossConnectRetrieveResponse, virtual_cross_connect, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.virtual_cross_connects.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        virtual_cross_connect = client.virtual_cross_connects.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(VirtualCrossConnectUpdateResponse, virtual_cross_connect, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        virtual_cross_connect = client.virtual_cross_connects.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            primary_cloud_ip="169.254.0.2",
            primary_enabled=True,
            primary_routing_announcement=False,
            secondary_cloud_ip="169.254.0.4",
            secondary_enabled=True,
            secondary_routing_announcement=False,
        )
        assert_matches_type(VirtualCrossConnectUpdateResponse, virtual_cross_connect, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.virtual_cross_connects.with_raw_response.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_cross_connect = response.parse()
        assert_matches_type(VirtualCrossConnectUpdateResponse, virtual_cross_connect, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.virtual_cross_connects.with_streaming_response.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_cross_connect = response.parse()
            assert_matches_type(VirtualCrossConnectUpdateResponse, virtual_cross_connect, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.virtual_cross_connects.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        virtual_cross_connect = client.virtual_cross_connects.list()
        assert_matches_type(VirtualCrossConnectListResponse, virtual_cross_connect, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        virtual_cross_connect = client.virtual_cross_connects.list(
            filter={"network_id": "6a09cdc3-8948-47f0-aa62-74ac943d6c58"},
            page={
                "number": 1,
                "size": 1,
            },
        )
        assert_matches_type(VirtualCrossConnectListResponse, virtual_cross_connect, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.virtual_cross_connects.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_cross_connect = response.parse()
        assert_matches_type(VirtualCrossConnectListResponse, virtual_cross_connect, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.virtual_cross_connects.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_cross_connect = response.parse()
            assert_matches_type(VirtualCrossConnectListResponse, virtual_cross_connect, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        virtual_cross_connect = client.virtual_cross_connects.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(VirtualCrossConnectDeleteResponse, virtual_cross_connect, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.virtual_cross_connects.with_raw_response.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_cross_connect = response.parse()
        assert_matches_type(VirtualCrossConnectDeleteResponse, virtual_cross_connect, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.virtual_cross_connects.with_streaming_response.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_cross_connect = response.parse()
            assert_matches_type(VirtualCrossConnectDeleteResponse, virtual_cross_connect, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.virtual_cross_connects.with_raw_response.delete(
                "",
            )


class TestAsyncVirtualCrossConnects:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        virtual_cross_connect = await async_client.virtual_cross_connects.create(
            bgp_asn=1234,
            cloud_provider="aws",
            cloud_provider_region="us-east-1",
            network_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            primary_cloud_account_id="123456789012",
            region_code="ashburn-va",
        )
        assert_matches_type(VirtualCrossConnectCreateResponse, virtual_cross_connect, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        virtual_cross_connect = await async_client.virtual_cross_connects.create(
            bgp_asn=1234,
            cloud_provider="aws",
            cloud_provider_region="us-east-1",
            network_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            primary_cloud_account_id="123456789012",
            region_code="ashburn-va",
            bandwidth_mbps=50,
            name="test interface",
            primary_bgp_key="yFV4wEPtPVPfDUGLWiyQzwga",
            primary_cloud_ip="169.254.0.2",
            primary_telnyx_ip="169.254.0.1",
            secondary_bgp_key="ge1lONeK9RcA83uuWaw9DvZy",
            secondary_cloud_account_id="",
            secondary_cloud_ip="169.254.0.4",
            secondary_telnyx_ip="169.254.0.3",
        )
        assert_matches_type(VirtualCrossConnectCreateResponse, virtual_cross_connect, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.virtual_cross_connects.with_raw_response.create(
            bgp_asn=1234,
            cloud_provider="aws",
            cloud_provider_region="us-east-1",
            network_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            primary_cloud_account_id="123456789012",
            region_code="ashburn-va",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_cross_connect = await response.parse()
        assert_matches_type(VirtualCrossConnectCreateResponse, virtual_cross_connect, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.virtual_cross_connects.with_streaming_response.create(
            bgp_asn=1234,
            cloud_provider="aws",
            cloud_provider_region="us-east-1",
            network_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            primary_cloud_account_id="123456789012",
            region_code="ashburn-va",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_cross_connect = await response.parse()
            assert_matches_type(VirtualCrossConnectCreateResponse, virtual_cross_connect, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        virtual_cross_connect = await async_client.virtual_cross_connects.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(VirtualCrossConnectRetrieveResponse, virtual_cross_connect, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.virtual_cross_connects.with_raw_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_cross_connect = await response.parse()
        assert_matches_type(VirtualCrossConnectRetrieveResponse, virtual_cross_connect, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.virtual_cross_connects.with_streaming_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_cross_connect = await response.parse()
            assert_matches_type(VirtualCrossConnectRetrieveResponse, virtual_cross_connect, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.virtual_cross_connects.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        virtual_cross_connect = await async_client.virtual_cross_connects.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(VirtualCrossConnectUpdateResponse, virtual_cross_connect, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        virtual_cross_connect = await async_client.virtual_cross_connects.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            primary_cloud_ip="169.254.0.2",
            primary_enabled=True,
            primary_routing_announcement=False,
            secondary_cloud_ip="169.254.0.4",
            secondary_enabled=True,
            secondary_routing_announcement=False,
        )
        assert_matches_type(VirtualCrossConnectUpdateResponse, virtual_cross_connect, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.virtual_cross_connects.with_raw_response.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_cross_connect = await response.parse()
        assert_matches_type(VirtualCrossConnectUpdateResponse, virtual_cross_connect, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.virtual_cross_connects.with_streaming_response.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_cross_connect = await response.parse()
            assert_matches_type(VirtualCrossConnectUpdateResponse, virtual_cross_connect, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.virtual_cross_connects.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        virtual_cross_connect = await async_client.virtual_cross_connects.list()
        assert_matches_type(VirtualCrossConnectListResponse, virtual_cross_connect, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        virtual_cross_connect = await async_client.virtual_cross_connects.list(
            filter={"network_id": "6a09cdc3-8948-47f0-aa62-74ac943d6c58"},
            page={
                "number": 1,
                "size": 1,
            },
        )
        assert_matches_type(VirtualCrossConnectListResponse, virtual_cross_connect, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.virtual_cross_connects.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_cross_connect = await response.parse()
        assert_matches_type(VirtualCrossConnectListResponse, virtual_cross_connect, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.virtual_cross_connects.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_cross_connect = await response.parse()
            assert_matches_type(VirtualCrossConnectListResponse, virtual_cross_connect, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        virtual_cross_connect = await async_client.virtual_cross_connects.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(VirtualCrossConnectDeleteResponse, virtual_cross_connect, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.virtual_cross_connects.with_raw_response.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_cross_connect = await response.parse()
        assert_matches_type(VirtualCrossConnectDeleteResponse, virtual_cross_connect, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.virtual_cross_connects.with_streaming_response.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_cross_connect = await response.parse()
            assert_matches_type(VirtualCrossConnectDeleteResponse, virtual_cross_connect, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.virtual_cross_connects.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["RunListParams", "Page"]


class RunListParams(TypedDict, total=False):
    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[size], page[number]
    """

    status: str
    """Filter runs by execution status (pending, running, completed, failed, timeout)"""


class Page(TypedDict, total=False):
    number: int
    """Page number to retrieve (1-based indexing)"""

    size: int
    """Number of test runs to return per page (1-100)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .reserved_phone_number import ReservedPhoneNumber

__all__ = ["NumberReservation"]


class NumberReservation(BaseModel):
    id: Optional[str] = None

    created_at: Optional[datetime] = None
    """An ISO 8901 datetime string denoting when the numbers reservation was created."""

    customer_reference: Optional[str] = None
    """A customer reference string for customer look ups."""

    phone_numbers: Optional[List[ReservedPhoneNumber]] = None

    record_type: Optional[str] = None

    status: Optional[Literal["pending", "success", "failure"]] = None
    """The status of the entire reservation."""

    updated_at: Optional[datetime] = None
    """An ISO 8901 datetime string for when the number reservation was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, TypeAlias, TypedDict

__all__ = [
    "AddressListParams",
    "Filter",
    "FilterAddressBook",
    "FilterCustomerReference",
    "FilterCustomerReferenceUnionMember1",
    "FilterStreetAddress",
    "Page",
]


class AddressListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[customer_reference][eq],
    filter[customer_reference][contains], filter[used_as_emergency],
    filter[street_address][contains], filter[address_book][eq]
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[number], page[size]
    """

    sort: Literal["created_at", "first_name", "last_name", "business_name", "street_address"]
    """Specifies the sort order for results.

    By default sorting direction is ascending. To have the results sorted in
    descending order add the <code> -</code> prefix.<br/><br/> That is: <ul>

      <li>
        <code>street_address</code>: sorts the result by the
        <code>street_address</code> field in ascending order.
      </li>

      <li>
        <code>-street_address</code>: sorts the result by the
        <code>street_address</code> field in descending order.
      </li>
    </ul> <br/> If not given, results are sorted by <code>created_at</code> in descending order.
    """


class FilterAddressBook(TypedDict, total=False):
    eq: str
    """
    If present, only returns results with the <code>address_book</code> flag equal
    to the given value.
    """


class FilterCustomerReferenceUnionMember1(TypedDict, total=False):
    contains: str
    """Partial match for customer_reference. Matching is not case-sensitive."""

    eq: str
    """Exact match for customer_reference."""


FilterCustomerReference: TypeAlias = Union[str, FilterCustomerReferenceUnionMember1]


class FilterStreetAddress(TypedDict, total=False):
    contains: str
    """
    If present, addresses with <code>street_address</code> containing the given
    value will be returned. Matching is not case-sensitive. Requires at least three
    characters.
    """


class Filter(TypedDict, total=False):
    address_book: FilterAddressBook

    customer_reference: FilterCustomerReference
    """
    If present, addresses with <code>customer_reference</code> containing the given
    value will be returned. Matching is not case-sensitive.
    """

    street_address: FilterStreetAddress

    used_as_emergency: str
    """
    If set as 'true', only addresses used as the emergency address for at least one
    active phone-number will be returned. When set to 'false', the opposite happens:
    only addresses not used as the emergency address from phone-numbers will be
    returned.
    """


class Page(TypedDict, total=False):
    number: int
    """The page number to load"""

    size: int
    """The size of the page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from ..pagination_meta import PaginationMeta
from .porting_loa_configuration import PortingLoaConfiguration

__all__ = ["LoaConfigurationListResponse"]


class LoaConfigurationListResponse(BaseModel):
    data: Optional[List[PortingLoaConfiguration]] = None

    meta: Optional[PaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal

import httpx

from ..types import dialogflow_connection_create_params, dialogflow_connection_update_params
from .._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.dialogflow_connection_create_response import DialogflowConnectionCreateResponse
from ..types.dialogflow_connection_update_response import DialogflowConnectionUpdateResponse
from ..types.dialogflow_connection_retrieve_response import DialogflowConnectionRetrieveResponse

__all__ = ["DialogflowConnectionsResource", "AsyncDialogflowConnectionsResource"]


class DialogflowConnectionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DialogflowConnectionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return DialogflowConnectionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DialogflowConnectionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return DialogflowConnectionsResourceWithStreamingResponse(self)

    def create(
        self,
        connection_id: str,
        *,
        service_account: Dict[str, object],
        conversation_profile_id: str | Omit = omit,
        dialogflow_api: Literal["cx", "es"] | Omit = omit,
        environment: str | Omit = omit,
        location: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DialogflowConnectionCreateResponse:
        """
        Save Dialogflow Credentiails to Telnyx, so it can be used with other Telnyx
        services.

        Args:
          service_account: The JSON map to connect your Dialoglow account.

          conversation_profile_id: The id of a configured conversation profile on your Dialogflow account. (If you
              use Dialogflow CX, this param is required)

          dialogflow_api: Determine which Dialogflow will be used.

          environment: Which Dialogflow environment will be used.

          location: The region of your agent is. (If you use Dialogflow CX, this param is required)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        return self._post(
            f"/dialogflow_connections/{connection_id}",
            body=maybe_transform(
                {
                    "service_account": service_account,
                    "conversation_profile_id": conversation_profile_id,
                    "dialogflow_api": dialogflow_api,
                    "environment": environment,
                    "location": location,
                },
                dialogflow_connection_create_params.DialogflowConnectionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DialogflowConnectionCreateResponse,
        )

    def retrieve(
        self,
        connection_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DialogflowConnectionRetrieveResponse:
        """
        Return details of the Dialogflow connection associated with the given
        CallControl connection.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        return self._get(
            f"/dialogflow_connections/{connection_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DialogflowConnectionRetrieveResponse,
        )

    def update(
        self,
        connection_id: str,
        *,
        service_account: Dict[str, object],
        conversation_profile_id: str | Omit = omit,
        dialogflow_api: Literal["cx", "es"] | Omit = omit,
        environment: str | Omit = omit,
        location: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DialogflowConnectionUpdateResponse:
        """
        Updates a stored Dialogflow Connection.

        Args:
          service_account: The JSON map to connect your Dialoglow account.

          conversation_profile_id: The id of a configured conversation profile on your Dialogflow account. (If you
              use Dialogflow CX, this param is required)

          dialogflow_api: Determine which Dialogflow will be used.

          environment: Which Dialogflow environment will be used.

          location: The region of your agent is. (If you use Dialogflow CX, this param is required)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        return self._put(
            f"/dialogflow_connections/{connection_id}",
            body=maybe_transform(
                {
                    "service_account": service_account,
                    "conversation_profile_id": conversation_profile_id,
                    "dialogflow_api": dialogflow_api,
                    "environment": environment,
                    "location": location,
                },
                dialogflow_connection_update_params.DialogflowConnectionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DialogflowConnectionUpdateResponse,
        )

    def delete(
        self,
        connection_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Deletes a stored Dialogflow Connection.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/dialogflow_connections/{connection_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncDialogflowConnectionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDialogflowConnectionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDialogflowConnectionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDialogflowConnectionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncDialogflowConnectionsResourceWithStreamingResponse(self)

    async def create(
        self,
        connection_id: str,
        *,
        service_account: Dict[str, object],
        conversation_profile_id: str | Omit = omit,
        dialogflow_api: Literal["cx", "es"] | Omit = omit,
        environment: str | Omit = omit,
        location: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DialogflowConnectionCreateResponse:
        """
        Save Dialogflow Credentiails to Telnyx, so it can be used with other Telnyx
        services.

        Args:
          service_account: The JSON map to connect your Dialoglow account.

          conversation_profile_id: The id of a configured conversation profile on your Dialogflow account. (If you
              use Dialogflow CX, this param is required)

          dialogflow_api: Determine which Dialogflow will be used.

          environment: Which Dialogflow environment will be used.

          location: The region of your agent is. (If you use Dialogflow CX, this param is required)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        return await self._post(
            f"/dialogflow_connections/{connection_id}",
            body=await async_maybe_transform(
                {
                    "service_account": service_account,
                    "conversation_profile_id": conversation_profile_id,
                    "dialogflow_api": dialogflow_api,
                    "environment": environment,
                    "location": location,
                },
                dialogflow_connection_create_params.DialogflowConnectionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DialogflowConnectionCreateResponse,
        )

    async def retrieve(
        self,
        connection_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DialogflowConnectionRetrieveResponse:
        """
        Return details of the Dialogflow connection associated with the given
        CallControl connection.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        return await self._get(
            f"/dialogflow_connections/{connection_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DialogflowConnectionRetrieveResponse,
        )

    async def update(
        self,
        connection_id: str,
        *,
        service_account: Dict[str, object],
        conversation_profile_id: str | Omit = omit,
        dialogflow_api: Literal["cx", "es"] | Omit = omit,
        environment: str | Omit = omit,
        location: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DialogflowConnectionUpdateResponse:
        """
        Updates a stored Dialogflow Connection.

        Args:
          service_account: The JSON map to connect your Dialoglow account.

          conversation_profile_id: The id of a configured conversation profile on your Dialogflow account. (If you
              use Dialogflow CX, this param is required)

          dialogflow_api: Determine which Dialogflow will be used.

          environment: Which Dialogflow environment will be used.

          location: The region of your agent is. (If you use Dialogflow CX, this param is required)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        return await self._put(
            f"/dialogflow_connections/{connection_id}",
            body=await async_maybe_transform(
                {
                    "service_account": service_account,
                    "conversation_profile_id": conversation_profile_id,
                    "dialogflow_api": dialogflow_api,
                    "environment": environment,
                    "location": location,
                },
                dialogflow_connection_update_params.DialogflowConnectionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DialogflowConnectionUpdateResponse,
        )

    async def delete(
        self,
        connection_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Deletes a stored Dialogflow Connection.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/dialogflow_connections/{connection_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class DialogflowConnectionsResourceWithRawResponse:
    def __init__(self, dialogflow_connections: DialogflowConnectionsResource) -> None:
        self._dialogflow_connections = dialogflow_connections

        self.create = to_raw_response_wrapper(
            dialogflow_connections.create,
        )
        self.retrieve = to_raw_response_wrapper(
            dialogflow_connections.retrieve,
        )
        self.update = to_raw_response_wrapper(
            dialogflow_connections.update,
        )
        self.delete = to_raw_response_wrapper(
            dialogflow_connections.delete,
        )


class AsyncDialogflowConnectionsResourceWithRawResponse:
    def __init__(self, dialogflow_connections: AsyncDialogflowConnectionsResource) -> None:
        self._dialogflow_connections = dialogflow_connections

        self.create = async_to_raw_response_wrapper(
            dialogflow_connections.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            dialogflow_connections.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            dialogflow_connections.update,
        )
        self.delete = async_to_raw_response_wrapper(
            dialogflow_connections.delete,
        )


class DialogflowConnectionsResourceWithStreamingResponse:
    def __init__(self, dialogflow_connections: DialogflowConnectionsResource) -> None:
        self._dialogflow_connections = dialogflow_connections

        self.create = to_streamed_response_wrapper(
            dialogflow_connections.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            dialogflow_connections.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            dialogflow_connections.update,
        )
        self.delete = to_streamed_response_wrapper(
            dialogflow_connections.delete,
        )


class AsyncDialogflowConnectionsResourceWithStreamingResponse:
    def __init__(self, dialogflow_connections: AsyncDialogflowConnectionsResource) -> None:
        self._dialogflow_connections = dialogflow_connections

        self.create = async_to_streamed_response_wrapper(
            dialogflow_connections.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            dialogflow_connections.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            dialogflow_connections.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            dialogflow_connections.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .upload import Upload
from ..._models import BaseModel
from ..external_voice_integrations_pagination_meta import ExternalVoiceIntegrationsPaginationMeta

__all__ = ["UploadListResponse"]


class UploadListResponse(BaseModel):
    data: Optional[List[Upload]] = None

    meta: Optional[ExternalVoiceIntegrationsPaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["IPListParams", "Filter", "Page"]


class IPListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[connection_id], filter[ip_address], filter[port]
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[size], page[number]
    """


class Filter(TypedDict, total=False):
    connection_id: str
    """ID of the IP Connection to which this IP should be attached."""

    ip_address: str
    """IP adddress represented by this resource."""

    port: int
    """Port to use when connecting to this IP."""


class Page(TypedDict, total=False):
    number: int
    """The page number to load"""

    size: int
    """The size of the page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["InvoiceListParams", "Page"]


class InvoiceListParams(TypedDict, total=False):
    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[number], page[size]
    """

    sort: Literal["period_start", "-period_start"]
    """Specifies the sort order for results."""


class Page(TypedDict, total=False):
    number: int
    """The page number to load"""

    size: int
    """The size of the page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    OAuthClientListResponse,
    OAuthClientCreateResponse,
    OAuthClientUpdateResponse,
    OAuthClientRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestOAuthClients:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        oauth_client = client.oauth_clients.create(
            allowed_grant_types=["client_credentials"],
            allowed_scopes=["admin"],
            client_type="public",
            name="My OAuth client",
        )
        assert_matches_type(OAuthClientCreateResponse, oauth_client, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        oauth_client = client.oauth_clients.create(
            allowed_grant_types=["client_credentials"],
            allowed_scopes=["admin"],
            client_type="public",
            name="My OAuth client",
            logo_uri="https://example.com",
            policy_uri="https://example.com",
            redirect_uris=["https://example.com"],
            require_pkce=True,
            tos_uri="https://example.com",
        )
        assert_matches_type(OAuthClientCreateResponse, oauth_client, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.oauth_clients.with_raw_response.create(
            allowed_grant_types=["client_credentials"],
            allowed_scopes=["admin"],
            client_type="public",
            name="My OAuth client",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        oauth_client = response.parse()
        assert_matches_type(OAuthClientCreateResponse, oauth_client, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.oauth_clients.with_streaming_response.create(
            allowed_grant_types=["client_credentials"],
            allowed_scopes=["admin"],
            client_type="public",
            name="My OAuth client",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            oauth_client = response.parse()
            assert_matches_type(OAuthClientCreateResponse, oauth_client, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        oauth_client = client.oauth_clients.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(OAuthClientRetrieveResponse, oauth_client, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.oauth_clients.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        oauth_client = response.parse()
        assert_matches_type(OAuthClientRetrieveResponse, oauth_client, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.oauth_clients.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            oauth_client = response.parse()
            assert_matches_type(OAuthClientRetrieveResponse, oauth_client, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.oauth_clients.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        oauth_client = client.oauth_clients.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(OAuthClientUpdateResponse, oauth_client, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        oauth_client = client.oauth_clients.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            allowed_grant_types=["client_credentials"],
            allowed_scopes=["admin"],
            logo_uri="https://example.com",
            name="name",
            policy_uri="https://example.com",
            redirect_uris=["https://example.com"],
            require_pkce=True,
            tos_uri="https://example.com",
        )
        assert_matches_type(OAuthClientUpdateResponse, oauth_client, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.oauth_clients.with_raw_response.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        oauth_client = response.parse()
        assert_matches_type(OAuthClientUpdateResponse, oauth_client, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.oauth_clients.with_streaming_response.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            oauth_client = response.parse()
            assert_matches_type(OAuthClientUpdateResponse, oauth_client, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.oauth_clients.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        oauth_client = client.oauth_clients.list()
        assert_matches_type(OAuthClientListResponse, oauth_client, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        oauth_client = client.oauth_clients.list(
            filter_allowed_grant_types_contains="client_credentials",
            filter_client_id="filter[client_id]",
            filter_client_type="confidential",
            filter_name="filter[name]",
            filter_name_contains="filter[name][contains]",
            filter_verified=True,
            page_number=1,
            page_size=1,
        )
        assert_matches_type(OAuthClientListResponse, oauth_client, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.oauth_clients.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        oauth_client = response.parse()
        assert_matches_type(OAuthClientListResponse, oauth_client, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.oauth_clients.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            oauth_client = response.parse()
            assert_matches_type(OAuthClientListResponse, oauth_client, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        oauth_client = client.oauth_clients.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert oauth_client is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.oauth_clients.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        oauth_client = response.parse()
        assert oauth_client is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.oauth_clients.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            oauth_client = response.parse()
            assert oauth_client is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.oauth_clients.with_raw_response.delete(
                "",
            )


class TestAsyncOAuthClients:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        oauth_client = await async_client.oauth_clients.create(
            allowed_grant_types=["client_credentials"],
            allowed_scopes=["admin"],
            client_type="public",
            name="My OAuth client",
        )
        assert_matches_type(OAuthClientCreateResponse, oauth_client, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        oauth_client = await async_client.oauth_clients.create(
            allowed_grant_types=["client_credentials"],
            allowed_scopes=["admin"],
            client_type="public",
            name="My OAuth client",
            logo_uri="https://example.com",
            policy_uri="https://example.com",
            redirect_uris=["https://example.com"],
            require_pkce=True,
            tos_uri="https://example.com",
        )
        assert_matches_type(OAuthClientCreateResponse, oauth_client, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.oauth_clients.with_raw_response.create(
            allowed_grant_types=["client_credentials"],
            allowed_scopes=["admin"],
            client_type="public",
            name="My OAuth client",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        oauth_client = await response.parse()
        assert_matches_type(OAuthClientCreateResponse, oauth_client, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.oauth_clients.with_streaming_response.create(
            allowed_grant_types=["client_credentials"],
            allowed_scopes=["admin"],
            client_type="public",
            name="My OAuth client",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            oauth_client = await response.parse()
            assert_matches_type(OAuthClientCreateResponse, oauth_client, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        oauth_client = await async_client.oauth_clients.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(OAuthClientRetrieveResponse, oauth_client, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.oauth_clients.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        oauth_client = await response.parse()
        assert_matches_type(OAuthClientRetrieveResponse, oauth_client, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.oauth_clients.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            oauth_client = await response.parse()
            assert_matches_type(OAuthClientRetrieveResponse, oauth_client, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.oauth_clients.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        oauth_client = await async_client.oauth_clients.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(OAuthClientUpdateResponse, oauth_client, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        oauth_client = await async_client.oauth_clients.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            allowed_grant_types=["client_credentials"],
            allowed_scopes=["admin"],
            logo_uri="https://example.com",
            name="name",
            policy_uri="https://example.com",
            redirect_uris=["https://example.com"],
            require_pkce=True,
            tos_uri="https://example.com",
        )
        assert_matches_type(OAuthClientUpdateResponse, oauth_client, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.oauth_clients.with_raw_response.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        oauth_client = await response.parse()
        assert_matches_type(OAuthClientUpdateResponse, oauth_client, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.oauth_clients.with_streaming_response.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            oauth_client = await response.parse()
            assert_matches_type(OAuthClientUpdateResponse, oauth_client, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.oauth_clients.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        oauth_client = await async_client.oauth_clients.list()
        assert_matches_type(OAuthClientListResponse, oauth_client, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        oauth_client = await async_client.oauth_clients.list(
            filter_allowed_grant_types_contains="client_credentials",
            filter_client_id="filter[client_id]",
            filter_client_type="confidential",
            filter_name="filter[name]",
            filter_name_contains="filter[name][contains]",
            filter_verified=True,
            page_number=1,
            page_size=1,
        )
        assert_matches_type(OAuthClientListResponse, oauth_client, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.oauth_clients.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        oauth_client = await response.parse()
        assert_matches_type(OAuthClientListResponse, oauth_client, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.oauth_clients.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            oauth_client = await response.parse()
            assert_matches_type(OAuthClientListResponse, oauth_client, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        oauth_client = await async_client.oauth_clients.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert oauth_client is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.oauth_clients.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        oauth_client = await response.parse()
        assert oauth_client is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.oauth_clients.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            oauth_client = await response.parse()
            assert oauth_client is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.oauth_clients.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .porting_order import PortingOrder

__all__ = ["PortingOrderUpdateResponse", "Meta"]


class Meta(BaseModel):
    phone_numbers_url: Optional[str] = None
    """Link to list all phone numbers"""


class PortingOrderUpdateResponse(BaseModel):
    data: Optional[PortingOrder] = None

    meta: Optional[Meta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["PhoneNumberExtensionListParams", "Filter", "Page", "Sort"]


class PhoneNumberExtensionListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[porting_phone_number_id]
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[size], page[number]
    """

    sort: Sort
    """Consolidated sort parameter (deepObject style). Originally: sort[value]"""


class Filter(TypedDict, total=False):
    porting_phone_number_id: str
    """Filter results by porting phone number id"""


class Page(TypedDict, total=False):
    number: int
    """The page number to load"""

    size: int
    """The size of the page"""


class Sort(TypedDict, total=False):
    value: Literal["-created_at", "created_at"]
    """Specifies the sort order for results.

    If not given, results are sorted by created_at in descending order
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["PortingOrderMessaging"]


class PortingOrderMessaging(BaseModel):
    enable_messaging: Optional[bool] = None
    """
    Indicates whether Telnyx will port messaging capabilities from the losing
    carrier. If false, any messaging capabilities will stay with their current
    provider.
    """

    messaging_capable: Optional[bool] = None
    """Indicates whether the porting order can also port messaging capabilities."""

    messaging_port_completed: Optional[bool] = None
    """Indicates whether the messaging porting has been completed."""

    messaging_port_status: Optional[
        Literal["not_applicable", "pending", "activating", "exception", "canceled", "partial_port_complete", "ported"]
    ] = None
    """The current status of the messaging porting."""
